//---------------------------------------------------------------------------
// WinActiv
//
// by Rik Blok
// January 8, 1999
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "win32x.h"		// for GetWindowTextX()
//---------------------------------------------------------------------------
USERES("WinActiv.res");
USEUNIT("\My Documents\CBuilder\Library\win32x.cpp");
//---------------------------------------------------------------------------
AnsiString globalName;
HWND handle;
typedef enum {MatchAll=0, MatchLeft, MatchRight, MatchMiddle} MatchType;
MatchType MatchPos;

//---------------------------------------------------------------------------
BOOL CALLBACK ListWindows(HWND hWnd, LPARAM lparam)
/*
	EnumWindowsCallback function to compare window.
    Uses global variables and saves window in handle.
*/
{
	// if no window specified then match first window enumerated
	if (globalName.IsEmpty()) {	// empty globalName always matches
    	handle = hWnd;
        return false;	// found, don't continue
    }

    // else, if text to match longer than window title then next
    AnsiString t = GetWindowTextX(hWnd);
    if (globalName.Length() > t.Length())
    	return true;	// not found, continue

    // else, if title perfect match then found
    if (globalName==t) {
    	handle = hWnd;
        return false;	// found, don't continue
    }

    // else, if matching middle look for text in caption
    if (MatchPos==MatchMiddle) {
    	if (t.Pos(globalName)>0) {
        	handle = hWnd;
            return false;	// found, don't continue
        } else
        	return true;	// not found, continue
	}

    // else, match left or right
    int index;
    int count;
    switch (MatchPos) {
        case MatchLeft:
        	index = 1;
            count = globalName.Length();
            break;
        case MatchRight:
        	index = 1 + t.Length() - globalName.Length();
            count = globalName.Length();
    }
	if (globalName==t.SubString(index,count)) {
    	handle = hWnd;
        return false;	// found, don't continue
    } else
    	return true;	// not found, continue
}
//---------------------------------------------------------------------------
HWND FindWindowBA(AnsiString WindowName)
/*
	As FindWindow() but supports wildcard before and/or after text.
    Eg. FindWindowBA("*Notepad");
    Returns first window found matching criteria.
*/
{
	// set global variables before iterating through windows
    // strip off wildcards
    MatchPos = MatchAll;	// default
    if (!WindowName.IsEmpty()) {
    	int last = WindowName.Length();
    	MatchPos = MatchType((WindowName[last]=='*') + 2*(WindowName[1]=='*'));
        if (MatchPos==MatchRight || MatchPos==MatchMiddle)
        	WindowName = WindowName.SubString(2,(last--)-1);
        if (MatchPos==MatchLeft || MatchPos==MatchMiddle)
        	WindowName = WindowName.SubString(1,(last--)-1);
    }
	globalName = WindowName;
    handle = NULL;
    // ok, let's find it!
	EnumWindows((WNDENUMPROC)ListWindows,0);
    return handle;
}
//---------------------------------------------------------------------------//---------------------------------------------------------------------------
void usage(void)
// print usage message
{
	printf("\nWinActiv - activates a window specified on the command-line");
    printf("\n");
    printf("\nUsage:");
    printf("\n\tWinActiv \"window title\"");
    printf("\n");
    printf("\nwhere \"window title\" is the caption of the window to switch to.");
    printf("\nWildcards \"*\" are accepted at the beginning or end of the title");
    printf("\nto match substrings.");
    printf("\nIf the window is not found, an error code of 1 is returned.");
    printf("\n");
}
//---------------------------------------------------------------------------//---------------------------------------------------------------------------
int main(int argc, char **argv)
{
	if (argc!=2) {	// must have exactly one parameter
    	usage();
        return 1;
    }
    // else
    AnsiString caption = argv[1];
    FindWindowBA(caption);
    if (handle==NULL) return 1;
    // else
    SetForegroundWindow(handle);
	return 0;
}
//---------------------------------------------------------------------------
